#! /usr/local/bin/apl --script
⍝ ********************************************************************
⍝ apl_kbd.apl Workspace to enable the windows key for apl characters.
⍝ Copyright (C) 2020 Bill Daly

⍝ This program is free software: you can redistribute it and/or modify
⍝ it under the terms of the GNU General Public License as published by
⍝ the Free Software Foundation, either version 3 of the License, or
⍝ (at your option) any later version.

⍝ This program is distributed in the hope that it will be useful,
⍝ but WITHOUT ANY WARRANTY; without even the implied warranty of
⍝ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
⍝ GNU General Public License for more details.

⍝ You should have received a copy of the GNU General Public License
⍝ along with this program.  If not, see <http://www.gnu.org/licenses/>.

⍝ ********************************************************************
⍝ Workspace requires package x11-xkb-utils.
⍝ ********************************************************************

)copy_once 5 FILE_IO
)copy_once 1 utl
)copy_once 1 lex
)copy_once 1 import
)copy_once 1 export

∇kbd_on shift_key;env
  ⍝ Function sets the 'Windows key' as the apl shift key.
  →(∧/kbd_opt_help=(⍴kbd_opt_help)↑shift_key)/help
  utl∆es (~kbd_valid_options utl∆member shift_key)/shift_key,' is not a valid shift key'
  →(kbd_has_apl_dir)/real_work
  utl∆es(~1+ 511 FIO∆mkdir kbd_storage)/'ERROR CREATING ',kbd_storage
real_work:
  env←kbd_environment
  env kbd_save_env kbd_storage,'/kbd_params.lex'
  →(env lex∆haskey 'variant')/0
  shift_key kbd_set env lex∆assign 'variant' 'apl'
  →0
help:
  ⍞←kbd_help
∇

∇kbd_off;env
  ⍝ Function resets the 'Windows key'
  env ←kbd_read_env kbd_storage,'/kbd_params.lex'
  →(~env lex∆haskey 'variant')/set
  env←env lex∆drop 'variant'
set:
  kbd_set env
∇

∇rs←kbd_environment;tmp
  ⍝ Function calls os command setxkbmap to obtain information about
  ⍝ keyboard setup. Function returns a lexicon
  utl∆es (~kbd_in_path 'setxkbmap')/'X.org command ''setxkbmap'' not available. Is Xorg running?'
  tmp←FIO∆pipefrom 'setxkbmap -query'
  rs←':' utl∆split ¨ ⎕tc[3] utl∆split tmp
  rs←⊃(∊2=⍴¨rs)/rs
∇

∇ grp kbd_set args;variant;sink
  ⍝ Function to change the keyboard environment for apl
  ⍎(2=⎕nc 'grp')/'args←args lex∆assign ''option'' grp'
  cmd←'setxkbmap -rules ',args lex∆lookup 'rules'
  cmd←cmd,' -model ',args lex∆lookup 'model'
  cmd←cmd,' -layout ',(args lex∆lookup 'layout')
  →(~args lex∆haskey 'variant')/ex
  cmd←cmd,',',variant←args lex∆lookup 'variant'
  cmd←cmd,' -variant ',variant
  cmd←cmd,' -option -option ',args lex∆lookup 'option'
ex:
  sink←FIO∆pipefrom cmd
∇

∇ env kbd_save_env fname
  ⍝ Function to save the current keyboard mapping
  env export∆array fname
∇

∇ env←kbd_read_env fname
  ⍝ Function to read the keyboard mapping saved by kbd_save_env
  env←import∆file fname
∇

∇b← kbd_in_path fnName;dirList;fileList;lb;ix
⍝ Function test whether fnName exists in the current PATH.
dirList←':' utl∆split 2⊃,⎕env 'PATH'
lb←((⍴dirList)⍴st),ed
ix←1
st:
→((0=1↑⍴fileList)∨0=⍴⍴fileList←(FIO∆read_directory ix⊃dirList))/tst
→(b←fileList[;5] utl∆member fnName)/ed
tst:
→lb[ix←ix+1]
ed:
→0
∇

∇h←kbd_home_dir
  ⍝ Function returns the users home directory.
  h←2⊃,⎕env 'HOME'
∇


∇b←kbd_has_apl_dir
  ⍝ Function tests for existance of .gnu-apl directory
  b←(FIO∆read_directory kbd_home_dir)[;5] utl∆member '.gnu-apl'
∇

∇ rs←kbd_xmodmap_on
  ⍝ Function to call xmodmap
  rs←FIO∆pipefrom 'xmodmap ',kbd_home_dir,'/wslib1/ancillary/Dyalog.xmodmap'
∇

∇Z←kbd⍙metadata
Z←0 2⍴⍬
Z←Z⍪'Author'          'Bill Daly'
Z←Z⍪'BugEmail'        'bugs@dalywebandedit.com'
Z←Z⍪'Documentation'   'file'
Z←Z⍪'Download'        'https://sourceforge.net/projects/apl-library/'
Z←Z⍪'License'         'GNU GPL 3'
Z←Z⍪'Portability'     ''
Z←Z⍪'Provides'        ''
Z←Z⍪'Requires'        'FILE_IO; apl-library'
Z←Z⍪'Version'         '0 1 0'

kbd_storage←kbd_home_dir,'/.gnu-apl'


kbd_opt_win←'grp:win_switch'	 ⍝ Menu key
kbd_opt_rctrl←'grp:rctrl_switch' ⍝ Right control key
kbd_opt_ralt←'grp:switch'	 ⍝ Right Alt key
kbd_opt_lalt←'grp:lswitch'	 ⍝ Left Alt key
kbd_valid_options ←  kbd_opt_win kbd_opt_rctrl kbd_opt_ralt kbd_opt_lalt

kbd_opt_help←'help'

kbd_help←⊃⎕inp 'end_of_help'
This workspace will reset the X windows keyboard mapping to include a
fourth shift key so that one may type APL operators and functions.

kbd_on shift_key does the remapping.  shift_key is on of:

   kbd_opt_win to use the Window key to the left of the space bar;
   kbd_opt_rctrl to use the ctrl key to the right of the space bar;
   kbd_opt_ralt to use the alt key to the right of the space bar;
   kbd_opt_lalt to use the alt key to the left of the space bar;


The function kbd_off resets the keyboard mapping to what was in effect
before executing kbd_off

Note this is a session wide process. That is turning on the APL
key mapping will be true in all windows opened in the current session.
Logging out should reset the keyboard mapping.
end_of_help
