#! /usr/local/bin/apl --script
⍝ ********************************************************************
⍝ lex_assoc.apl An GNU APL workspace to implement a lexicon as an
⍝ associative  array
⍝ Copyright (C) 2020 Bill Daly

⍝ This program is free software: you can redistribute it and/or modify
⍝ it under the terms of the GNU General Public License as published by
⍝ the Free Software Foundation, either version 3 of the License, or
⍝ (at your option) any later version.

⍝ This program is distributed in the hope that it will be useful,
⍝ but WITHOUT ANY WARRANTY; without even the implied warranty of
⍝ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
⍝ GNU General Public License for more details.

⍝ You should have received a copy of the GNU General Public License
⍝ along with this program.  If not, see <http://www.gnu.org/licenses/>.

⍝ ********************************************************************

)COPY_ONCE 1 utl

∇lex←lexa∆init n
  ⍝ Function to initiate a lexicon; n is the initial size of the
  ⍝ associative array.
  lex←38 ⎕cr n
∇

∇nx←lx lexa∆assign item
  ⍝ Function to assign a value to a key where larg is the lexicon and
  ⍝ rarg is a key value pair. Pair will be added if the key is not in
  ⍝ the lexicon.
  utl∆es (2≠⍴item←,item)/'RIGHT ARGUMENT MUST BE A TWO ELEMENT VECTOR'
  lx[1⊃item]←2⊃item
  nx←lx
∇

∇new←lex lexa∆drop key;sink
  ⍝ Function to drop a key value pair
  utl∆es (~lex lexa∆haskey key)/key,' NOT DEFINED IN LEXICON.'
  ⎕ex 'lex.',key
  new←lex
∇

∇b←lexa∆is lex;active
  ⍝ Predicate returns true if argument is in fact a lexicon
  ⍎(2≠⍴⍴lex)/'b←0 ◊ →0'
  ⍎(2≠¯1↑⍴lex)/'b←0 ◊ →0'
  b←''⍴(⍴lex[;1])≠1↑⍴lex
∇

∇bool←lex lexa∆haskey key;active
  ⍝ Function searches for key in lexicon lex and returns true if
  ⍝ found.
  utl∆es (~lexa∆is lex)/'LEFT ARGUMENT MUST BE A LEXICON'
  utl∆es (~utl∆stringp key)/'RIGHT ARGUMENT MUST BE A CHARACTER VECTOR'
  bool←''⍴(⍴lex[;1])≥lex[;1] utl∆listSearch key
∇

∇b←lexa∆isempty lex
  ⍝ Predicate returns true is the lexicon has no entries
  utl∆es (~lexa∆is lex)/'LEFT ARGUMENT MUST BE A LEXICON'
  b←''⍴0=⍴lex[;1]
∇

∇keys←lexa∆keys lx;active
  ⍝ Function returns a list of keys in a lexicon
  keys←lx[;1]
∇

∇vals←lexa∆values lx;tmp
  ⍝ returns the values in a lexicon
  tmp←lexa∆keys lx
  vals←{lx lexa∆lookup ⍵}¨tmp
∇

∇value←lex lexa∆lookup key;active
  ⍝ Function returns the value of key in a lexicon. If the key does
  ⍝ not exist it returns ⍬
  ⍎(~lex lexa∆haskey key)/'value←⍬ ◊ →0'
  value←lex[key]
∇

∇new←lexa∆from_alist list;keys;values;vx;sink
  ⍝ Function to generate a lexicon from an alist.  An a list is a list
  ⍝ of alternating keys and their related values.
  utl∆es (1≠⍴⍴list)/'AN ALIST IS A VECTOR OF KEY VALUE PAIRS'
  utl∆es (0≠2|⍴list)/'AN ALIST IS A VECTOR OF KEY VALUE PAIRS'
  new←38 ⎕cr (((⍴list)÷2),2)⍴list
∇

∇new←delims lexa∆from_delim txt;sink;tmp
  ⍝ Function creates a lexicon from a delimited string. Delims is a
  ⍝ vector of two characters. The first is the character found in the
  ⍝ items delimited by the second.
  new←38 ⎕cr ⊃(1⊃delims) utl∆split ¨ (2⊃delims) utl∆split txt
∇

∇Z←lexa⍙metadata
  Z←0 2⍴⍬
  Z←Z⍪'Author'          'Bill Daly'
  Z←Z⍪'BugEmail'        'bugs@dalywebandedit.com'
  Z←Z⍪'Documentation'   'eoc/apl-library.info, hopefully'
  Z←Z⍪'Download'        'https://sourceforge.net/p/apl-library/code/ci/master/tree/lex.apl'
  Z←Z⍪'License'         'GPL v3.0'
  Z←Z⍪'Portability'     'L1'
  Z←Z⍪'Provides'        ''
  Z←Z⍪'Requires'        ''
  Z←Z⍪'Version'                  '0 2 0'
  Z←Z⍪'Last update'         '2020-12-28'
∇

lexa∆message←⎕inp 'eom'

This workspace is an experiment to provide the lex API for GNU APL's
structured variables, an exaperimental feature. Use info to read File:
apl.info, Node: Section 3.35, Prev: Section 3.34, Up: Chapter 3.

Use function utl∆helpFns to read the purpose of each function in the
lex API. All functions have a 'lexa∆' prefix.
eom


⎕lx←'utl∆display lexa∆message'

